@@ -1,19 +1,42 @@
 // * This makes emacs happy -*-Mode: C++;-*-
-#ifndef _CURSESW_H
-#define _CURSESW_H
+/****************************************************************************
+ * Copyright (c) 1998-2001,2004 Free Software Foundation, Inc.              *
+ *                                                                          *
+ * Permission is hereby granted, free of charge, to any person obtaining a  *
+ * copy of this software and associated documentation files (the            *
+ * "Software"), to deal in the Software without restriction, including      *
+ * without limitation the rights to use, copy, modify, merge, publish,      *
+ * distribute, distribute with modifications, sublicense, and/or sell       *
+ * copies of the Software, and to permit persons to whom the Software is    *
+ * furnished to do so, subject to the following conditions:                 *
+ *                                                                          *
+ * The above copyright notice and this permission notice shall be included  *
+ * in all copies or substantial portions of the Software.                   *
+ *                                                                          *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
+ * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
+ * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
+ * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
+ * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
+ *                                                                          *
+ * Except as contained in this notice, the name(s) of the above copyright   *
+ * holders shall not be used in advertising or otherwise to promote the     *
+ * sale, use or other dealings in this Software without prior written       *
+ * authorization.                                                           *
+ ****************************************************************************/
 
-// $Id: cursesw.h,v 1.17 1999/09/11 23:28:08 tom Exp $
+#ifndef NCURSES_CURSESW_H_incl
+#define NCURSES_CURSESW_H_incl 1
+
+// $Id: cursesw.h,v 1.30 2004/02/08 00:11:54 tom Exp $
 
 #include <etip.h>
-#include <stdio.h>
 #include <stdarg.h>
-#ifdef __MWERKS__
-/* This is a bogus check, stringstream is actually ANSI C++ standard,
- * but old compilers like GCC don't have it, and new compilers like Metrowerks
- * don't have strstream
- */
-#include <sstream>
-#else
+#include <stdio.h>
+
+#if HAVE_STRSTREAM_H && (USE_STRSTREAM_VSCAN||USE_STRSTREAM_VSCAN_CAST)
 #include <strstream.h>
 #endif
 
@@ -50,8 +73,7 @@ inline int UNDEF(insdelln)(int n)  { return insdelln(n); }
 #endif
 
 #ifdef addstr
-/* The (char*) cast is to hack around missing const's */
-inline int UNDEF(addstr)(const char * str)  { return addstr((char*)str); }
+inline int UNDEF(addstr)(const char * str)  { return addstr(str); }
 #undef addstr
 #define addstr UNDEF(addstr)
 #endif
@@ -75,14 +97,14 @@ inline chtype UNDEF(attrset)(chtype at) { return attrset(at); }
 #endif
 
 #ifdef color_set
-inline chtype UNDEF(color_set)(short p,void* opts) { return color_set(p,opts); }
+inline chtype UNDEF(color_set)(short p, void* opts) { return color_set(p, opts); }
 #undef color_set
 #define color_set UNDEF(color_set)
 #endif
 
 #ifdef border
 inline int UNDEF(border)(chtype ls, chtype rs, chtype ts, chtype bs, chtype tl, chtype tr, chtype bl, chtype br)
-{ return border(ls,rs,ts,bs,tl,tr,bl,br); }
+{ return border(ls, rs, ts, bs, tl, tr, bl, br); }
 #undef border
 #define border UNDEF(border)
 #endif
@@ -94,15 +116,15 @@ inline int UNDEF(box)(WINDOW *win, int v, int h) { return box(win, v, h); }
 #endif
 
 #ifdef mvwhline
-inline int UNDEF(mvwhline)(WINDOW *win,int y,int x,chtype c,int n) {
-  return mvwhline(win,y,x,c,n); }
+inline int UNDEF(mvwhline)(WINDOW *win, int y, int x, chtype c, int n) {
+  return mvwhline(win, y, x, c, n); }
 #undef mvwhline
 #define mvwhline UNDEF(mvwhline)
 #endif
 
 #ifdef mvwvline
-inline int UNDEF(mvwvline)(WINDOW *win,int y,int x,chtype c,int n) {
-  return mvwvline(win,y,x,c,n); }
+inline int UNDEF(mvwvline)(WINDOW *win, int y, int x, chtype c, int n) {
+  return mvwvline(win, y, x, c, n); }
 #undef mvwvline
 #define mvwvline UNDEF(mvwvline)
 #endif
@@ -118,7 +140,7 @@ inline int UNDEF(clearok)(WINDOW* win, bool bf)  { return clearok(win, bf); }
 #undef clearok
 #define clearok UNDEF(clearok)
 #else
-extern "C" int clearok(WINDOW*, bool);
+extern "C" NCURSES_IMPEXP int NCURSES_API clearok(WINDOW*, bool);
 #endif
 
 #ifdef clrtobot
@@ -152,7 +174,7 @@ inline int UNDEF(erase)()  { return erase(); }
 #endif
 
 #ifdef flushok
-inline int UNDEF(flushok)(WINDOW* _win, bool _bf)  { 
+inline int UNDEF(flushok)(WINDOW* _win, bool _bf)  {
   return flushok(_win, _bf); }
 #undef flushok
 #define flushok UNDEF(flushok)
@@ -179,82 +201,82 @@ inline int UNDEF(instr)(char *_str)  { return instr(_str); }
 #endif
 
 #ifdef innstr
-inline int UNDEF(innstr)(char *_str, int n)  { return innstr(_str,n); }
+inline int UNDEF(innstr)(char *_str, int n)  { return innstr(_str, n); }
 #undef innstr
 #define innstr UNDEF(innstr)
 #endif
 
 #ifdef mvwinnstr
-inline int UNDEF(mvwinnstr)(WINDOW *win, int y, int x, char *_str, int n) { 
-  return mvwinnstr(win,y,x,_str,n); }
+inline int UNDEF(mvwinnstr)(WINDOW *win, int y, int x, char *_str, int n) {
+  return mvwinnstr(win, y, x, _str, n); }
 #undef mvwinnstr
 #define mvwinnstr UNDEF(mvwinnstr)
 #endif
 
 #ifdef mvinnstr
-inline int UNDEF(mvinnstr)(int y, int x, char *_str, int n) { 
-  return mvinnstr(y,x,_str,n); }
+inline int UNDEF(mvinnstr)(int y, int x, char *_str, int n) {
+  return mvinnstr(y, x, _str, n); }
 #undef mvinnstr
 #define mvinnstr UNDEF(mvinnstr)
 #endif
 
 #ifdef winsstr
-inline int UNDEF(winsstr)(WINDOW *w, const char *_str)  { 
-  return winsstr(w,_str); }
+inline int UNDEF(winsstr)(WINDOW *w, const char *_str)  {
+  return winsstr(w, _str); }
 #undef winsstr
 #define winsstr UNDEF(winsstr)
 #endif
 
 #ifdef mvwinsstr
-inline int UNDEF(mvwinsstr)(WINDOW *w, int y, int x,  const char *_str)  { 
-  return mvwinsstr(w,y,x,_str); }
+inline int UNDEF(mvwinsstr)(WINDOW *w, int y, int x,  const char *_str)  {
+  return mvwinsstr(w, y, x, _str); }
 #undef mvwinsstr
 #define mvwinsstr UNDEF(mvwinsstr)
 #endif
 
 #ifdef insstr
-inline int UNDEF(insstr)(const char *_str)  { 
+inline int UNDEF(insstr)(const char *_str)  {
   return insstr(_str); }
 #undef insstr
 #define insstr UNDEF(insstr)
 #endif
 
 #ifdef mvinsstr
-inline int UNDEF(mvinsstr)(int y, int x,const char *_str)  { 
-  return mvinsstr(y,x,_str); }
+inline int UNDEF(mvinsstr)(int y, int x, const char *_str)  {
+  return mvinsstr(y, x, _str); }
 #undef mvinsstr
 #define mvinsstr UNDEF(mvinsstr)
 #endif
 
 #ifdef insnstr
-inline int UNDEF(insnstr)(const char *_str, int n)  { 
-  return insnstr(_str,n); }
+inline int UNDEF(insnstr)(const char *_str, int n)  {
+  return insnstr(_str, n); }
 #undef insnstr
 #define insnstr UNDEF(insnstr)
 #endif
 
 #ifdef mvwinsnstr
-inline int UNDEF(mvwinsnstr)(WINDOW *w, int y, int x,const char *_str, int n) {
-  return mvwinsnstr(w,y,x,_str,n); }
+inline int UNDEF(mvwinsnstr)(WINDOW *w, int y, int x, const char *_str, int n) {
+  return mvwinsnstr(w, y, x, _str, n); }
 #undef mvwinsnstr
 #define mvwinsnstr UNDEF(mvwinsnstr)
 #endif
 
 #ifdef mvinsnstr
-inline int UNDEF(mvinsnstr)(int y, int x,const char *_str, int n) {
-  return mvinsnstr(y,x,_str,n); }
+inline int UNDEF(mvinsnstr)(int y, int x, const char *_str, int n) {
+  return mvinsnstr(y, x, _str, n); }
 #undef mvinsnstr
 #define mvinsnstr UNDEF(mvinsnstr)
 #endif
 
 #ifdef getnstr
-inline int UNDEF(getnstr)(char *_str, int n)  { return getnstr(_str,n); }
+inline int UNDEF(getnstr)(char *_str, int n)  { return getnstr(_str, n); }
 #undef getnstr
 #define getnstr UNDEF(getnstr)
 #endif
 
 #ifdef getyx
-inline void UNDEF(getyx)(const WINDOW* win, int& y, int& x) { 
+inline void UNDEF(getyx)(const WINDOW* win, int& y, int& x) {
   getyx(win, y, x); }
 #undef getyx
 #define getyx UNDEF(getyx)
@@ -301,7 +323,7 @@ inline int UNDEF(leaveok)(WINDOW* win, bool bf)  { return leaveok(win, bf); }
 #undef leaveok
 #define leaveok UNDEF(leaveok)
 #else
-extern "C" int leaveok(WINDOW* win, bool bf);
+extern "C" NCURSES_IMPEXP int NCURSES_API leaveok(WINDOW* win, bool bf);
 #endif
 
 #ifdef move
@@ -340,9 +362,9 @@ inline int UNDEF(scrollok)(WINDOW* win, bool bf)  { return scrollok(win, bf); }
 #define scrollok UNDEF(scrollok)
 #else
 #if	defined(__NCURSES_H)
-extern "C" int scrollok(WINDOW*, bool);
+extern "C" NCURSES_IMPEXP int NCURSES_API scrollok(WINDOW*, bool);
 #else
-extern "C" int scrollok(WINDOW*, char);
+extern "C" NCURSES_IMPEXP int NCURSES_API scrollok(WINDOW*, char);
 #endif
 #endif
 
@@ -365,8 +387,8 @@ inline int UNDEF(standout)()  { return standout(); }
 #endif
 
 #ifdef subpad
-inline WINDOW *UNDEF(subpad)(WINDOW *p, int l, int c, int y, int x) 
-{ return derwin(p,l,c,y,x); }
+inline WINDOW *UNDEF(subpad)(WINDOW *p, int l, int c, int y, int x)
+{ return derwin(p, l, c, y, x); }
 #undef subpad
 #define subpad UNDEF(subpad)
 #endif
@@ -379,7 +401,7 @@ inline void UNDEF(timeout)(int delay) { timeout(delay); }
 
 #ifdef touchline
 inline int UNDEF(touchline)(WINDOW *win, int s, int c)
-{ return touchline(win,s,c); }
+{ return touchline(win, s, c); }
 #undef touchline
 #define touchline UNDEF(touchline)
 #endif
@@ -434,24 +456,24 @@ inline int UNDEF(wattroff)(WINDOW *win, int att) { return wattroff(win, att); }
 #endif
 
 #ifdef chgat
-inline int UNDEF(chgat)(int n,attr_t attr, short color, const void *opts) { 
-  return chgat(n,attr,color,opts); }
+inline int UNDEF(chgat)(int n, attr_t attr, short color, const void *opts) {
+  return chgat(n, attr, color, opts); }
 #undef chgat
 #define chgat UNDEF(chgat)
 #endif
 
 #ifdef mvchgat
 inline int UNDEF(mvchgat)(int y, int x, int n,
-			  attr_t attr, short color, const void *opts) { 
-  return mvchgat(y,x,n,attr,color,opts); }
+			  attr_t attr, short color, const void *opts) {
+  return mvchgat(y, x, n, attr, color, opts); }
 #undef mvchgat
 #define mvchgat UNDEF(mvchgat)
 #endif
 
 #ifdef mvwchgat
 inline int UNDEF(mvwchgat)(WINDOW *win, int y, int x, int n,
-			   attr_t attr, short color, const void *opts) { 
-  return mvwchgat(win,y,x,n,attr,color,opts); }
+			   attr_t attr, short color, const void *opts) {
+  return mvwchgat(win, y, x, n, attr, color, opts); }
 #undef mvwchgat
 #define mvwchgat UNDEF(mvwchgat)
 #endif
@@ -491,21 +513,21 @@ inline int UNDEF(mvwaddchstr)(WINDOW *win, int y, int x, chtype *str)
 
 #ifdef addnstr
 inline int UNDEF(addnstr)(const char *str, int n)
-{ return addnstr((char*)str, n); }
+{ return addnstr(str, n); }
 #undef addnstr
 #define addnstr UNDEF(addnstr)
 #endif
 
 #ifdef mvwaddnstr
 inline int UNDEF(mvwaddnstr)(WINDOW *win, int y, int x, const char *str, int n)
-{ return mvwaddnstr(win, y, x, (char*)str, n); }
+{ return mvwaddnstr(win, y, x, str, n); }
 #undef mvwaddnstr
 #define mvwaddnstr UNDEF(mvwaddnstr)
 #endif
 
 #ifdef mvwaddstr
 inline int UNDEF(mvwaddstr)(WINDOW *win, int y, int x, const char * str)
-{ return mvwaddstr(win, y, x, (char*)str); }
+{ return mvwaddstr(win, y, x, str); }
 #undef mvwaddstr
 #define mvwaddstr UNDEF(mvwaddstr)
 #endif
@@ -525,20 +547,20 @@ inline int UNDEF(mvwgetch)(WINDOW *win, int y, int x) { return mvwgetch(win, y,
 
 #ifdef mvwgetstr
 inline int UNDEF(mvwgetstr)(WINDOW *win, int y, int x, char *str)
-{return mvwgetstr(win,y,x, str);}
+{return mvwgetstr(win, y, x, str);}
 #undef mvwgetstr
 #define mvwgetstr UNDEF(mvwgetstr)
 #endif
 
 #ifdef mvwgetnstr
 inline int UNDEF(mvwgetnstr)(WINDOW *win, int y, int x, char *str, int n)
-{return mvwgetnstr(win,y,x, str,n);}
+{return mvwgetnstr(win, y, x, str, n);}
 #undef mvwgetnstr
 #define mvwgetnstr UNDEF(mvwgetnstr)
 #endif
 
 #ifdef mvwinch
-inline chtype UNDEF(mvwinch)(WINDOW *win, int y, int x) { 
+inline chtype UNDEF(mvwinch)(WINDOW *win, int y, int x) {
   return mvwinch(win, y, x);}
 #undef mvwinch
 #define mvwinch UNDEF(mvwinch)
@@ -560,14 +582,14 @@ inline int UNDEF(mvaddch)(int y, int x, chtype ch)
 
 #ifdef mvaddnstr
 inline int UNDEF(mvaddnstr)(int y, int x, const char *str, int n)
-{ return mvaddnstr(y, x, (char*)str, n); }
+{ return mvaddnstr(y, x, str, n); }
 #undef mvaddnstr
 #define mvaddnstr UNDEF(mvaddnstr)
 #endif
 
 #ifdef mvaddstr
 inline int UNDEF(mvaddstr)(int y, int x, const char * str)
-{ return mvaddstr(y, x, (char*)str); }
+{ return mvaddstr(y, x, str); }
 #undef mvaddstr
 #define mvaddstr UNDEF(mvaddstr)
 #endif
@@ -592,7 +614,7 @@ inline int UNDEF(mvgetstr)(int y, int x, char *str) {return mvgetstr(y, x, str);
 
 #ifdef mvgetnstr
 inline int UNDEF(mvgetnstr)(int y, int x, char *str, int n) {
-  return mvgetnstr(y, x, str,n);}
+  return mvgetnstr(y, x, str, n);}
 #undef mvgetnstr
 #define mvgetnstr UNDEF(mvgetnstr)
 #endif
@@ -671,18 +693,18 @@ inline void UNDEF(bkgdset)(chtype ch) { bkgdset(ch); }
  *
  */
 
-class NCursesWindow
+class NCURSES_IMPEXP NCursesWindow
 {
   friend class NCursesMenu;
   friend class NCursesForm;
-  
+
 private:
   static bool    b_initialized;
   static void    initialize();
-  static int     ripoff_init(WINDOW *,int);
+  static int     ripoff_init(WINDOW *, int);
+
+  void           init();
 
-  void           init(); 
-  
   short          getcolor(int getback) const;
 
   static int     setpalette(short fore, short back, short pair);
@@ -700,15 +722,15 @@ protected:
   //   We rely on the c++ promise that
   //   all otherwise uninitialized
   //   static class vars are set to 0
-  
+
   WINDOW*        w;                // the curses WINDOW
-  
+
   bool           alloced;          // TRUE if we own the WINDOW
-  
+
   NCursesWindow* par;              // parent, if subwindow
   NCursesWindow* subwins;          // head of subwindows list
   NCursesWindow* sib;              // next subwindow of parent
-  
+
   void           kill_subwindows(); // disable all subwindows
   // Destroy all subwindows.
 
@@ -718,18 +740,18 @@ protected:
 
 public:
   NCursesWindow(WINDOW* &window);  // useful only for stdscr
-  
+
   NCursesWindow(int lines,         // number of lines
 		int cols,          // number of columns
 		int begin_y,       // line origin
 		int begin_x);      // col origin
-  
+
   NCursesWindow(NCursesWindow& par,// parent window
 		int lines,         // number of lines
 		int cols,          // number of columns
 		int begin_y,       // absolute or relative
 		int begin_x,       //   origins:
-		char absrel = 'a');// if `a', by & bx are
+		char absrel = 'a');// if `a', begin_y & begin_x are
   // absolute screen pos, else if `r', they are relative to par origin
 
   NCursesWindow(NCursesWindow& par,// parent window
@@ -751,31 +773,31 @@ public:
 			int (*init)(NCursesWindow& win));
   // This function is used to generate a window of ripped-of lines.
   // If the argument is positive, lines are removed from the top, if it
-  // is negative lines are removed from the bottom. This enhances the 
-  // lowlevel ripoffline() function because it uses the internal 
-  // implementation that allows to remove more than just a single line. 
+  // is negative lines are removed from the bottom. This enhances the
+  // lowlevel ripoffline() function because it uses the internal
+  // implementation that allows to remove more than just a single line.
   // This function must be called before any other ncurses function. The
-  // creation of the window is defered until ncurses gets initialized.
+  // creation of the window is deferred until ncurses gets initialized.
   // The initialization function is then called.
 
   // -------------------------------------------------------------------------
   // terminal status
   // -------------------------------------------------------------------------
-  int            lines() const { return LINES; }
+  int            lines() const { initialize(); return LINES; }
   // Number of lines on terminal, *not* window
 
-  int            cols() const { return COLS; }
+  int            cols() const { initialize(); return COLS; }
   // Number of cols  on terminal, *not* window
 
-  int            tabsize() const { return TABSIZE; }
+  int            tabsize() const { initialize(); return TABSIZE; }
   // Size of a tab on terminal, *not* window
 
   static int     NumberOfColors();
   // Number of available colors
 
-  int     colors() const { return NumberOfColors(); }
+  int            colors() const { return NumberOfColors(); }
   // Number of available colors
-  
+
   // -------------------------------------------------------------------------
   // window status
   // -------------------------------------------------------------------------
@@ -797,26 +819,26 @@ public:
   int            maxy() const { return w->_maxy; }
   // Largest y coord in window
 
-  short  getcolor() const;
+  short          getcolor() const;
   // Actual color pair
 
-  short  foreground() const { return getcolor(0); }
+  short          foreground() const { return getcolor(0); }
   // Actual foreground color
 
-  short  background() const { return getcolor(1); }
+  short          background() const { return getcolor(1); }
   // Actual background color
 
-  int    setpalette(short fore, short back);
+  int            setpalette(short fore, short back);
   // Set color palette entry
 
-  int    setcolor(short pair);
+  int            setcolor(short pair);
   // Set actually used palette entry
-  
+
   // -------------------------------------------------------------------------
   // window positioning
   // -------------------------------------------------------------------------
-  virtual int  mvwin(int begin_y, int begin_x) {
-    return ::mvwin(w,begin_y,begin_x); }
+  virtual int    mvwin(int begin_y, int begin_x) {
+    return ::mvwin(w, begin_y, begin_x); }
   // Move window to new position with the new position as top left corner.
   // This is virtual because it is redefined in NCursesPanel.
 
@@ -827,45 +849,44 @@ public:
   // Move cursor the this position
 
   void           getyx(int& y, int& x) const { ::getyx(w, y, x); }
-  // Get current position of the cursor 
+  // Get current position of the cursor
 
-  int            mvcur(int oldrow, int oldcol, int newrow, int newcol) const { 
+  int            mvcur(int oldrow, int oldcol, int newrow, int newcol) const {
     return ::mvcur(oldrow, oldcol, newrow, newcol); }
   // Perform lowlevel cursor motion that takes effect immediately.
-  
+
   // -------------------------------------------------------------------------
   // input
   // -------------------------------------------------------------------------
   int            getch() { return ::wgetch(w); }
   // Get a keystroke from the window.
 
-  int            getch(int y, int x) { return ::mvwgetch(w,y,x); }
+  int            getch(int y, int x) { return ::mvwgetch(w, y, x); }
   // Move cursor to position and get a keystroke from the window
 
-  int            getstr(char* str, int n=-1) { 
-    return ::wgetnstr(w, str,n); }
+  int            getstr(char* str, int n=-1) {
+    return ::wgetnstr(w, str, n); }
   // Read a series of characters into str until a newline or carriage return
   // is received. Read at most n characters. If n is negative, the limit is
   // ignored.
 
-  int            getstr(int y, int x, char* str, int n=-1) { 
-    return ::mvwgetnstr(w,y,x,str,n); }
+  int            getstr(int y, int x, char* str, int n=-1) {
+    return ::mvwgetnstr(w, y, x, str, n); }
   // Move the cursor to the requested position and then perform the getstr()
   // as described above.
 
-  int            instr(char *s, int n=-1) { return ::winnstr(w,s,n); }
+  int            instr(char *s, int n=-1) { return ::winnstr(w, s, n); }
   // Get a string of characters from the window into the buffer s. Retrieve
   // at most n characters, if n is negative retrieve all characters up to the
   // end of the current line. Attributes are stripped from the characters.
 
   int            instr(int y, int x, char *s, int n=-1) {
-    return ::mvwinnstr(w,y,x,s,n); }
+    return ::mvwinnstr(w, y, x, s, n); }
   // Move the cursor to the requested position and then perform the instr()
   // as described above.
 
   int            scanw(const char* fmt, ...)
-    // Perform a scanw function from the window. This only works if you're
-    // using the GNU C++ compiler.
+    // Perform a scanw function from the window.
 #if __GNUG__ >= 2
     __attribute__ ((format (scanf, 2, 3)));
 #else
@@ -874,40 +895,40 @@ public:
 
   int            scanw(int y, int x, const char* fmt, ...)
     // Move the cursor to the requested position and then perform a scanw
-    // from the window. This nly works if you're using the GNU C++ compiler.
+    // from the window.
 #if __GNUG__ >= 2
     __attribute__ ((format (scanf, 4, 5)));
 #else
   ;
 #endif
-  
+
   // -------------------------------------------------------------------------
   // output
   // -------------------------------------------------------------------------
   int            addch(const chtype ch) { return ::waddch(w, ch); }
-  // Put attributed character to the window. 
+  // Put attributed character to the window.
 
   int            addch(int y, int x, const chtype ch) {
-    return ::mvwaddch(w,y,x,ch); }
+    return ::mvwaddch(w, y, x, ch); }
   // Move cursor to the requested position and then put attributed character
   // to the window.
 
-  int            echochar(const chtype ch) { return ::wechochar(w,ch); }
+  int            echochar(const chtype ch) { return ::wechochar(w, ch); }
   // Put attributed character to the window and refresh it immediately.
 
-  int            addstr(const char* str, int n=-1) { 
-    return ::waddnstr(w, (char*)str,n); }
+  int            addstr(const char* str, int n=-1) {
+    return ::waddnstr(w, str, n); }
   // Write the string str to the window, stop writing if the terminating
-  // NUL or the limit n is reached. If n is negative, it is ignored. 
+  // NUL or the limit n is reached. If n is negative, it is ignored.
 
   int            addstr(int y, int x, const char * str, int n=-1) {
-    return ::mvwaddnstr(w,y,x,(char*)str,n); }
+    return ::mvwaddnstr(w, y, x, str, n); }
   // Move the cursor to the requested position and then perform the addstr
   // as described above.
 
   int            printw(const char* fmt, ...)
     // Do a formatted print to the window.
-#if __GNUG__ >= 2
+#if (__GNUG__ >= 2) && !defined(printf)
     __attribute__ ((format (printf, 2, 3)));
 #else
   ;
@@ -915,7 +936,7 @@ public:
 
   int            printw(int y, int x, const char * fmt, ...)
     // Move the cursor and then do a formatted print to the window.
-#if __GNUG__ >= 2
+#if (__GNUG__ >= 2) && !defined(printf)
     __attribute__ ((format (printf, 4, 5)));
 #else
   ;
@@ -924,7 +945,7 @@ public:
   chtype         inch() const { return ::winch(w); }
   // Retrieve attributed character under the current cursor position.
 
-  chtype         inch(int y, int x) { return ::mvwinch(w,y,x); }
+  chtype         inch(int y, int x) { return ::mvwinch(w, y, x); }
   // Move cursor to requested position and then retrieve attributed character
   // at this position.
 
@@ -933,31 +954,31 @@ public:
   // position.
 
   int            insch(int y, int x, chtype ch) {
-    return ::mvwinsch(w,y,x,(char)ch); }
+    return ::mvwinsch(w, y, x, ch); }
   // Move cursor to requested position and then insert the attributed
   // character before that position.
 
-  int            insertln() { return ::winsdelln(w,1); }
+  int            insertln() { return ::winsdelln(w, 1); }
   // Insert an empty line above the current line.
-  
-  int            insdelln(int n=1) { return ::winsdelln(w,n); }
+
+  int            insdelln(int n=1) { return ::winsdelln(w, n); }
   // If n>0 insert that many lines above the current line. If n<0 delete
   // that many lines beginning with the current line.
 
   int            insstr(const char *s, int n=-1) {
-    return ::winsnstr(w,s,n); }
+    return ::winsnstr(w, s, n); }
   // Insert the string into the window before the current cursor position.
   // Insert stops at end of string or when the limit n is reached. If n is
   // negative, it is ignored.
-  
+
   int            insstr(int y, int x, const char *s, int n=-1) {
-    return ::mvwinsnstr(w,y,x,s,n); }
+    return ::mvwinsnstr(w, y, x, s, n); }
   // Move the cursor to the requested position and then perform the insstr()
   // as described above.
 
   int            attron (chtype at) { return ::wattron (w, at); }
   // Switch on the window attributes;
-  
+
   int            attroff(chtype at) { return ::wattroff(w, (int) at); }
   // Switch off the window attributes;
 
@@ -968,15 +989,15 @@ public:
     return ::wcolor_set(w, color_pair_number, opts); }
   // Set the window color attribute;
 
-  int            chgat(int n,attr_t attr, short color, const void *opts=NULL) {
-    return ::wchgat(w,n,attr,color,opts); }
+  int            chgat(int n, attr_t attr, short color, const void *opts=NULL) {
+    return ::wchgat(w, n, attr, color, opts); }
   // Change the attributes of the next n characters in the current line. If
   // n is negative or greater than the number of remaining characters in the
   // line, the attributes will be changed up to the end of the line.
 
   int            chgat(int y, int x,
-		       int n,attr_t attr, short color, const void *opts=NULL) {
-    return ::mvwchgat(w,y,x,n,attr,color,opts); }
+		       int n, attr_t attr, short color, const void *opts=NULL) {
+    return ::mvwchgat(w, y, x, n, attr, color, opts); }
   // Move the cursor to the requested position and then perform chgat() as
   // described above.
 
@@ -986,27 +1007,27 @@ public:
   chtype         getbkgd() const { return ::getbkgd(w); }
   // Get current background setting.
 
-  int            bkgd(const chtype ch) { return ::wbkgd(w,ch); }
+  int            bkgd(const chtype ch) { return ::wbkgd(w, ch); }
   // Set the background property and apply it to the window.
 
-  void           bkgdset(chtype ch) { ::wbkgdset(w,ch); }
+  void           bkgdset(chtype ch) { ::wbkgdset(w, ch); }
   // Set the background property.
 
   // -------------------------------------------------------------------------
   // borders
   // -------------------------------------------------------------------------
-  int            box(chtype vert=0, chtype  hor=0) { 
-    return ::wborder(w, vert, vert, hor, hor, 0, 0 ,0, 0); }
+  int            box(chtype vert=0, chtype  hor=0) {
+    return ::wborder(w, vert, vert, hor, hor, 0, 0, 0, 0); }
   // Draw a box around the window with the given vertical and horizontal
-  // drawing characters. If you specifiy a zero as character, curses will try
+  // drawing characters. If you specify a zero as character, curses will try
   // to find a "nice" character.
 
   int            border(chtype left=0, chtype right=0,
 			chtype top =0, chtype bottom=0,
 			chtype top_left =0, chtype top_right=0,
 			chtype bottom_left =0, chtype bottom_right=0) {
-    return ::wborder(w,left,right,top,bottom,top_left,top_right,
-		     bottom_left,bottom_right); }
+    return ::wborder(w, left, right, top, bottom, top_left, top_right,
+		     bottom_left, bottom_right); }
   // Draw a border around the window with the given characters for the
   // various parts of the border. If you pass zero for a character, curses
   // will try to find "nice" characters.
@@ -1016,10 +1037,10 @@ public:
   // -------------------------------------------------------------------------
   int            hline(int len, chtype ch=0) { return ::whline(w, ch, len); }
   // Draw a horizontal line of len characters with the given character. If
-  // you pass zero for the character, curses will try to find a "nice" one. 
+  // you pass zero for the character, curses will try to find a "nice" one.
 
   int            hline(int y, int x, int len, chtype ch=0) {
-    return ::mvwhline(w,y,x,ch,len); }
+    return ::mvwhline(w, y, x, ch, len); }
   // Move the cursor to the requested position and then draw a horizontal line.
 
   int            vline(int len, chtype ch=0) { return ::wvline(w, ch, len); }
@@ -1027,9 +1048,9 @@ public:
   // you pass zero for the character, curses will try to find a "nice" one.
 
   int            vline(int y, int x, int len, chtype ch=0) {
-    return ::mvwvline(w,y,x,ch,len); }
+    return ::mvwvline(w, y, x, ch, len); }
   // Move the cursor to the requested position and then draw a vertical line.
-  
+
   // -------------------------------------------------------------------------
   // erasure
   // -------------------------------------------------------------------------
@@ -1050,19 +1071,19 @@ public:
   // Clear to the end of the line.
 
   int            delch() { return ::wdelch(w); }
-  // Delete character under the cursor. 
+  // Delete character under the cursor.
 
-  int            delch(int y, int x) { return ::mvwdelch(w,y,x); }
+  int            delch(int y, int x) { return ::mvwdelch(w, y, x); }
   // Move cursor to requested position and delete the character under the
   // cursor.
 
-  int            deleteln() { return ::winsdelln(w,-1); }
+  int            deleteln() { return ::winsdelln(w, -1); }
   // Delete the current line.
 
   // -------------------------------------------------------------------------
   // screen control
   // -------------------------------------------------------------------------
-  int            scroll(int amount=1) { return ::wscrl(w,amount); }
+  int            scroll(int amount=1) { return ::wscrl(w, amount); }
   // Scroll amount lines. If amount is positive, scroll up, otherwise
   // scroll down.
 
@@ -1072,7 +1093,7 @@ public:
   // at the bottom line.
 
   int            setscrreg(int from, int to) {
-    return ::wsetscrreg(w,from,to); }
+    return ::wsetscrreg(w, from, to); }
   // Define a soft scrolling region.
 
   int            idlok(bool bf) { return ::idlok(w, bf); }
@@ -1084,33 +1105,33 @@ public:
   // If bf is TRUE, use insert/delete character hardware support if possible.
   // Otherwise do it in software.
 
-  int            touchwin()   { return ::wtouchln(w,0,height(),1); }
+  int            touchwin()   { return ::wtouchln(w, 0, height(), 1); }
   // Mark the whole window as modified.
 
-  int            untouchwin() { return ::wtouchln(w,0,height(),0); }
+  int            untouchwin() { return ::wtouchln(w, 0, height(), 0); }
   // Mark the whole window as unmodified.
 
-  int            touchln(int s, int cnt, bool changed=TRUE) { 
-    return ::wtouchln(w,s,cnt,(int)(changed?1:0)); }
+  int            touchln(int s, int cnt, bool changed=TRUE) {
+    return ::wtouchln(w, s, cnt, (int)(changed?1:0)); }
   // Mark cnt lines beginning from line s as changed or unchanged, depending
   // on the value of the changed flag.
 
-  bool           is_linetouched(int line) const { 
-    return (::is_linetouched(w,line) ? TRUE:FALSE); }
-  // Return TRUE if line is marked as changed, FALSE otherwise 
+  bool           is_linetouched(int line) const {
+    return (::is_linetouched(w, line) ? TRUE:FALSE); }
+  // Return TRUE if line is marked as changed, FALSE otherwise
 
-  bool           is_wintouched() const { 
+  bool           is_wintouched() const {
     return (::is_wintouched(w) ? TRUE:FALSE); }
-  // Return TRUE if window is marked as changed, FALSE otherwise 
+  // Return TRUE if window is marked as changed, FALSE otherwise
 
   int            leaveok(bool bf) { return ::leaveok(w, bf); }
   // If bf is TRUE, curses will leave the cursor after an update whereever
   // it is after the update.
 
-  int            redrawln(int from, int n) { return ::wredrawln(w,from,n); }
+  int            redrawln(int from, int n) { return ::wredrawln(w, from, n); }
   // Redraw n lines starting from the requested line
 
-  int            redrawwin() { return ::wredrawln(w,0,height()); }
+  int            redrawwin() { return ::wredrawln(w, 0, height()); }
   // Redraw the whole window
 
   int            doupdate()  { return ::doupdate(); }
@@ -1125,23 +1146,23 @@ public:
   void           cursyncup() { ::wcursyncup(w); }
   // Position the cursor in all ancestor windows corresponding to our setting
 
-  int            syncok(bool bf) { return ::syncok(w,bf); }
+  int            syncok(bool bf) { return ::syncok(w, bf); }
   // If called with bf=TRUE, syncup() is called whenever the window is changed
 
 #ifndef _no_flushok
   int            flushok(bool bf) { return ::flushok(w, bf); }
 #endif
 
-  void           immedok(bool bf) { ::immedok(w,bf); }
+  void           immedok(bool bf) { ::immedok(w, bf); }
   // If called with bf=TRUE, any change in the window will cause an
   // automatic immediate refresh()
 
   int            keypad(bool bf) { return ::keypad(w, bf); }
   // If called with bf=TRUE, the application will interpret function keys.
 
-  int            meta(bool bf) { return ::meta(w,bf); }
+  int            meta(bool bf) { return ::meta(w, bf); }
   // If called with bf=TRUE, keys may generate 8-Bit characters. Otherwise
-  // 7-Bit characters are generated. 
+  // 7-Bit characters are generated.
 
   int            standout() { return ::wstandout(w); }
   // Enable "standout" attributes
@@ -1158,9 +1179,9 @@ public:
   // doupdate(). This is redefined in NCursesPanel.
 
   virtual int    noutrefresh() { return ::wnoutrefresh(w); }
-  // Propagate the changes in this window to the virtual screen. This is 
+  // Propagate the changes in this window to the virtual screen. This is
   // redefined in NCursesPanel.
-  
+
   // -------------------------------------------------------------------------
   // multiple window control
   // -------------------------------------------------------------------------
@@ -1176,8 +1197,8 @@ public:
 			 int sminrow, int smincol,
 			 int dminrow, int dmincol,
 			 int dmaxrow, int dmaxcol, bool overlay=TRUE) {
-    return ::copywin(w,win.w,sminrow,smincol,dminrow,dmincol,
-		     dmaxrow,dmaxcol,(int)(overlay?1:0)); }
+    return ::copywin(w, win.w, sminrow, smincol, dminrow, dmincol,
+		     dmaxrow, dmaxcol, (int)(overlay?1:0)); }
   // Overlay or overwrite the rectangle in win given by dminrow,dmincol,
   // dmaxrow,dmaxcol with the rectangle in this window beginning at
   // sminrow,smincol.
@@ -1207,7 +1228,7 @@ public:
 // -------------------------------------------------------------------------
 // We leave this here for compatibility reasons.
 // -------------------------------------------------------------------------
-class NCursesColorWindow : public NCursesWindow {
+class NCURSES_IMPEXP NCursesColorWindow : public NCursesWindow {
 public:
   NCursesColorWindow(WINDOW* &window)   // useful only for stdscr
     : NCursesWindow(window) {
@@ -1217,7 +1238,7 @@ public:
 		     int cols,          // number of columns
 		     int begin_y,       // line origin
 		     int begin_x)       // col origin
-    : NCursesWindow(lines,cols,begin_y,begin_x) {
+    : NCursesWindow(lines, cols, begin_y, begin_x) {
       useColors(); }
 
   NCursesColorWindow(NCursesWindow& par,// parent window
@@ -1226,10 +1247,10 @@ public:
 		     int begin_y,       // absolute or relative
 		     int begin_x,       //   origins:
 		     char absrel = 'a') // if `a', by & bx are
-    : NCursesWindow(par,lines,cols,     // absolute screen pos,
-		    begin_y,begin_x,    // else if `r', they are
-		    absrel ) {          // relative to par origin  
-      useColors(); }                      
+    : NCursesWindow(par, lines, cols,   // absolute screen pos,
+		    begin_y, begin_x,   // else if `r', they are
+		    absrel ) {          // relative to par origin
+      useColors(); }
 };
 
 // These enum definitions really belong inside the NCursesPad class, but only
@@ -1251,7 +1272,7 @@ public:
 // Pad Support. We allow an association of a pad with a "real" window
 // through which the pad may be viewed.
 // -------------------------------------------------------------------------
-class NCursesPad : public NCursesWindow {
+class NCURSES_IMPEXP NCursesPad : public NCursesWindow {
 private:
   NCursesWindow* viewWin;       // the "viewport" window
   NCursesWindow* viewSub;       // the "viewport" subwindow
@@ -1263,7 +1284,7 @@ protected:
 
   NCursesWindow* Win(void) const {
     // Get the window into which the pad should be copied (if any)
-    return (viewSub?viewSub:(viewWin?viewWin:NULL));
+    return (viewSub?viewSub:(viewWin?viewWin:0));
   }
 
   NCursesWindow* getWindow(void) const {
@@ -1298,10 +1319,10 @@ public:
 
   virtual ~NCursesPad() {}
 
-  int echochar(const chtype ch) { return ::pechochar(w,ch); }
+  int echochar(const chtype ch) { return ::pechochar(w, ch); }
   // Put the attributed character onto the pad and immediately do a
   // prefresh().
-  
+
   int refresh();
   // If a viewport is defined the pad is displayed in this window, otherwise
   // this is a noop.
@@ -1309,8 +1330,8 @@ public:
   int refresh(int pminrow, int pmincol,
 	      int sminrow, int smincol,
 	      int smaxrow, int smaxcol) {
-    return ::prefresh(w,pminrow,pmincol,
-		      sminrow,smincol,smaxrow,smaxcol);
+    return ::prefresh(w, pminrow, pmincol,
+		      sminrow, smincol, smaxrow, smaxcol);
   }
   // The coordinates sminrow,smincol,smaxrow,smaxcol describe a rectangle
   // on the screen. <b>refresh</b> copies a rectangle of this size beginning
@@ -1323,10 +1344,10 @@ public:
   int noutrefresh(int pminrow, int pmincol,
 		  int sminrow, int smincol,
 		  int smaxrow, int smaxcol) {
-    return ::pnoutrefresh(w,pminrow,pmincol,
-			  sminrow,smincol,smaxrow,smaxcol);
+    return ::pnoutrefresh(w, pminrow, pmincol,
+			  sminrow, smincol, smaxrow, smaxcol);
   }
-  // Does the same like refresh() but without calling doupdate().
+  // Does the same as refresh() but without calling doupdate().
 
   virtual void setWindow(NCursesWindow& view, int v_grid = 1, int h_grid = 1);
   // Add the window "view" as viewing window to the pad.
@@ -1343,15 +1364,15 @@ public:
 // A FramedPad is constructed always with a viewport window. This viewport
 // will be framed (by a box() command) and the interior of the box is the
 // viewport subwindow. On the frame we display scrollbar sliders.
-class NCursesFramedPad : public NCursesPad {
+class NCURSES_IMPEXP NCursesFramedPad : public NCursesPad {
 protected:
   virtual void OnOperation(int pad_req);
 
 public:
   NCursesFramedPad(NCursesWindow& win, int lines, int cols,
 		   int v_grid = 1, int h_grid = 1)
-    : NCursesPad(lines,cols) {
-    NCursesPad::setWindow(win,v_grid,h_grid);
+    : NCursesPad(lines, cols) {
+    NCursesPad::setWindow(win, v_grid, h_grid);
     NCursesPad::setSubWindow(*(new NCursesWindow(win)));
   }
   // Construct the FramedPad with the given Window win as viewport.
@@ -1372,4 +1393,4 @@ public:
 
 };
 
-#endif // _CURSESW_H
+#endif // NCURSES_CURSESW_H_incl
