@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2001,2002 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -39,7 +39,7 @@
 extern int malloc_errfd;	/* FIXME */
 #endif
 
-MODULE_ID("$Id: lib_freeall.c,v 1.20 2002/07/28 00:35:25 tom Exp $")
+MODULE_ID("$Id: lib_freeall.c,v 1.26 2003/12/27 18:21:57 tom Exp $")
 
 /*
  * Free all ncurses data.  This is used for testing only (there's no practical
@@ -51,8 +51,10 @@ _nc_freeall(void)
     WINDOWLIST *p, *q;
     char *s;
 
+    T((T_CALLED("_nc_freeall()")));
 #if NO_LEAKS
     _nc_free_tparm();
+    FreeAndNull(_nc_oldnums);
 #endif
     if (SP != 0) {
 	while (_nc_windows != 0) {
@@ -78,28 +80,38 @@ _nc_freeall(void)
 	delscreen(SP);
     }
 
-    if (cur_term != 0) {
-	_nc_free_termtype(&(cur_term->type));
-	free(cur_term);
-    }
+    del_curterm(cur_term);
+    _nc_free_entries(_nc_head);
 
     if ((s = _nc_home_terminfo()) != 0)
 	free(s);
+
+    (void) _nc_printf_string(0, 0);
 #ifdef TRACE
     (void) _nc_trace_buf(-1, 0);
 #endif
+
 #if HAVE_LIBDBMALLOC
     malloc_dump(malloc_errfd);
 #elif HAVE_LIBDMALLOC
 #elif HAVE_PURIFY
     purify_all_inuse();
 #endif
+    returnVoid;
 }
 
 NCURSES_EXPORT(void)
 _nc_free_and_exit(int code)
 {
+    char *last_setbuf = (SP != 0) ? SP->_setbuf : 0;
+
     _nc_freeall();
+#ifdef TRACE
+    trace(0);			/* close trace file, freeing its setbuf */
+    free(_nc_varargs("?", 0));
+#endif
+    fclose(stdout);
+    FreeIfNeeded(last_setbuf);
     exit(code);
 }
 
