@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1999 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1999-2000,2004 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -32,16 +32,11 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: version.c,v 1.2 1999/12/04 21:27:23 tom Exp $")
+MODULE_ID("$Id: version.c,v 1.5 2004/01/10 21:18:17 tom Exp $")
 
-const char *
+NCURSES_EXPORT(const char *)
 curses_version(void)
 {
-    static char my_version[80];
-
     T((T_CALLED("curses_version()")));
-    sprintf(my_version, "ncurses %s.%d",
-	NCURSES_VERSION,
-	NCURSES_VERSION_PATCH);
-    returnPtr(my_version);
+    returnPtr("ncurses " NCURSES_VERSION_STRING);
 }
