@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996-2003               *
  ****************************************************************************/
 
 /*
@@ -47,21 +48,24 @@
 #include <tic.h>
 #include <term_entry.h>
 
-MODULE_ID("$Id: alloc_entry.c,v 1.32 2000/03/12 00:16:31 tom Exp $")
+MODULE_ID("$Id: alloc_entry.c,v 1.40 2003/11/08 21:29:54 tom Exp $")
 
 #define ABSENT_OFFSET    -1
 #define CANCELLED_OFFSET -2
 
 #define MAX_STRTAB	4096	/* documented maximum entry size */
 
-static char stringbuf[MAX_STRTAB];	/* buffer for string capabilities */
+static char *stringbuf;		/* buffer for string capabilities */
 static size_t next_free;	/* next free character in stringbuf */
 
-void
+NCURSES_EXPORT(void)
 _nc_init_entry(TERMTYPE * const tp)
 /* initialize a terminal type data block */
 {
-    int i;
+    unsigned i;
+
+    if (stringbuf == 0)
+	stringbuf = (char *) malloc(MAX_STRTAB);
 
 #if NCURSES_XNAMES
     tp->num_Booleans = BOOLCOUNT;
@@ -90,7 +94,7 @@ _nc_init_entry(TERMTYPE * const tp)
     next_free = 0;
 }
 
-ENTRY *
+NCURSES_EXPORT(ENTRY *)
 _nc_copy_entry(ENTRY * oldp)
 {
     ENTRY *newp = typeCalloc(ENTRY, 1);
@@ -102,7 +106,7 @@ _nc_copy_entry(ENTRY * oldp)
     return newp;
 }
 
-char *
+NCURSES_EXPORT(char *)
 _nc_save_str(const char *const string)
 /* save a copy of string in the string buffer */
 {
@@ -118,14 +122,36 @@ _nc_save_str(const char *const string)
     return (stringbuf + old_next_free);
 }
 
-void
-_nc_wrap_entry(ENTRY * const ep)
+NCURSES_EXPORT(void)
+_nc_wrap_entry(ENTRY * const ep, bool copy_strings)
 /* copy the string parts to allocated storage, preserving pointers to it */
 {
     int offsets[MAX_ENTRY_SIZE / 2], useoffsets[MAX_USES];
-    int i, n;
+    unsigned i, n;
+    unsigned nuses = ep->nuses;
     TERMTYPE *tp = &(ep->tterm);
 
+    if (copy_strings) {
+	next_free = 0;		/* clear static storage */
+
+	/* copy term_names, Strings, uses */
+	tp->term_names = _nc_save_str(tp->term_names);
+	for_each_string(i, tp) {
+	    if (tp->Strings[i] != ABSENT_STRING &&
+		tp->Strings[i] != CANCELLED_STRING) {
+		tp->Strings[i] = _nc_save_str(tp->Strings[i]);
+	    }
+	}
+
+	for (i = 0; i < nuses; i++) {
+	    if (ep->uses[i].name == 0) {
+		ep->uses[i].name = _nc_save_str(ep->uses[i].name);
+	    }
+	}
+
+	free(tp->str_table);
+    }
+
     n = tp->term_names - stringbuf;
     for_each_string(i, &(ep->tterm)) {
 	if (tp->Strings[i] == ABSENT_STRING)
@@ -136,7 +162,7 @@ _nc_wrap_entry(ENTRY * const ep)
 	    offsets[i] = tp->Strings[i] - stringbuf;
     }
 
-    for (i = 0; i < ep->nuses; i++) {
+    for (i = 0; i < nuses; i++) {
 	if (ep->uses[i].name == 0)
 	    useoffsets[i] = ABSENT_OFFSET;
 	else
@@ -144,7 +170,7 @@ _nc_wrap_entry(ENTRY * const ep)
     }
 
     if ((tp->str_table = typeMalloc(char, next_free)) == (char *) 0)
-	  _nc_err_abort("Out of memory");
+	  _nc_err_abort(MSG_NO_MEMORY);
     (void) memcpy(tp->str_table, stringbuf, next_free);
 
     tp->term_names = tp->str_table + n;
@@ -158,23 +184,25 @@ _nc_wrap_entry(ENTRY * const ep)
     }
 
 #if NCURSES_XNAMES
-    if ((n = NUM_EXT_NAMES(tp)) != 0) {
-	unsigned length = 0;
-	for (i = 0; i < n; i++) {
-	    length += strlen(tp->ext_Names[i]) + 1;
-	    offsets[i] = tp->ext_Names[i] - stringbuf;
-	}
-	if ((tp->ext_str_table = typeMalloc(char, length)) == 0)
-	      _nc_err_abort("Out of memory");
-	for (i = 0, length = 0; i < n; i++) {
-	    tp->ext_Names[i] = tp->ext_str_table + length;
-	    strcpy(tp->ext_Names[i], stringbuf + offsets[i]);
-	    length += strlen(tp->ext_Names[i]) + 1;
+    if (!copy_strings) {
+	if ((n = NUM_EXT_NAMES(tp)) != 0) {
+	    unsigned length = 0;
+	    for (i = 0; i < n; i++) {
+		length += strlen(tp->ext_Names[i]) + 1;
+		offsets[i] = tp->ext_Names[i] - stringbuf;
+	    }
+	    if ((tp->ext_str_table = typeMalloc(char, length)) == 0)
+		  _nc_err_abort(MSG_NO_MEMORY);
+	    for (i = 0, length = 0; i < n; i++) {
+		tp->ext_Names[i] = tp->ext_str_table + length;
+		strcpy(tp->ext_Names[i], stringbuf + offsets[i]);
+		length += strlen(tp->ext_Names[i]) + 1;
+	    }
 	}
     }
 #endif
 
-    for (i = 0; i < ep->nuses; i++) {
+    for (i = 0; i < nuses; i++) {
 	if (useoffsets[i] == ABSENT_OFFSET)
 	    ep->uses[i].name = 0;
 	else
@@ -182,31 +210,35 @@ _nc_wrap_entry(ENTRY * const ep)
     }
 }
 
-void
+NCURSES_EXPORT(void)
 _nc_merge_entry(TERMTYPE * const to, TERMTYPE * const from)
 /* merge capabilities from `from' entry into `to' entry */
 {
-    int i;
+    unsigned i;
 
 #if NCURSES_XNAMES
     _nc_align_termtype(to, from);
 #endif
     for_each_boolean(i, from) {
-	int mergebool = from->Booleans[i];
+	if (to->Booleans[i] != CANCELLED_BOOLEAN) {
+	    int mergebool = from->Booleans[i];
 
-	if (mergebool == CANCELLED_BOOLEAN)
-	    to->Booleans[i] = FALSE;
-	else if (mergebool == TRUE)
-	    to->Booleans[i] = mergebool;
+	    if (mergebool == CANCELLED_BOOLEAN)
+		to->Booleans[i] = FALSE;
+	    else if (mergebool == TRUE)
+		to->Booleans[i] = mergebool;
+	}
     }
 
     for_each_number(i, from) {
-	int mergenum = from->Numbers[i];
+	if (to->Numbers[i] != CANCELLED_NUMERIC) {
+	    int mergenum = from->Numbers[i];
 
-	if (mergenum == CANCELLED_NUMERIC)
-	    to->Numbers[i] = ABSENT_NUMERIC;
-	else if (mergenum != ABSENT_NUMERIC)
-	    to->Numbers[i] = mergenum;
+	    if (mergenum == CANCELLED_NUMERIC)
+		to->Numbers[i] = ABSENT_NUMERIC;
+	    else if (mergenum != ABSENT_NUMERIC)
+		to->Numbers[i] = mergenum;
+	}
     }
 
     /*
@@ -215,11 +247,13 @@ _nc_merge_entry(TERMTYPE * const to, TERMTYPE * const from)
      * we ever want to deallocate entries.
      */
     for_each_string(i, from) {
-	char *mergestring = from->Strings[i];
+	if (to->Strings[i] != CANCELLED_STRING) {
+	    char *mergestring = from->Strings[i];
 
-	if (mergestring == CANCELLED_STRING)
-	    to->Strings[i] = ABSENT_STRING;
-	else if (mergestring != ABSENT_STRING)
-	    to->Strings[i] = mergestring;
+	    if (mergestring == CANCELLED_STRING)
+		to->Strings[i] = ABSENT_STRING;
+	    else if (mergestring != ABSENT_STRING)
+		to->Strings[i] = mergestring;
+	}
     }
 }
