@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -39,7 +39,7 @@
  *
  *	There is just one entry point:
  *
- *	char *captoinfo(n, s, parametrized)
+ *	char *_nc_captoinfo(n, s, parameterized)
  *
  *	Convert value s for termcap string capability named n into terminfo
  *	format.
@@ -92,7 +92,7 @@
 #include <ctype.h>
 #include <tic.h>
 
-MODULE_ID("$Id: captoinfo.c,v 1.37 2000/04/01 20:07:34 tom Exp $")
+MODULE_ID("$Id: captoinfo.c,v 1.45 2003/11/08 21:28:04 tom Exp $")
 
 #define MAX_PUSHED	16	/* max # args we can push onto the stack */
 
@@ -115,7 +115,7 @@ init_string(void)
     if (my_string == 0)
 	my_string = typeMalloc(char, my_length = 256);
     if (my_string == 0)
-	_nc_err_abort("Out of memory");
+	_nc_err_abort(MSG_NO_MEMORY);
 
     *my_string = '\0';
     return my_string;
@@ -129,7 +129,7 @@ save_string(char *d, const char *const s)
     if (need > my_length) {
 	my_string = (char *) realloc(my_string, my_length = (need + need));
 	if (my_string == 0)
-	    _nc_err_abort("Out of memory");
+	    _nc_err_abort(MSG_NO_MEMORY);
 	d = my_string + have;
     }
     (void) strcpy(d, s);
@@ -137,10 +137,10 @@ save_string(char *d, const char *const s)
 }
 
 static inline char *
-save_char(char *s, char c)
+save_char(char *s, int c)
 {
     static char temp[2];
-    temp[0] = c;
+    temp[0] = (char) c;
     return save_string(s, temp);
 }
 
@@ -194,7 +194,7 @@ cvtchar(register const char *sp)
 	case '2':
 	case '3':
 	    len = 1;
-	    while (isdigit(*sp)) {
+	    while (isdigit(UChar(*sp))) {
 		c = 8 * c + (*sp++ - '0');
 		len++;
 	    }
@@ -268,13 +268,16 @@ getparm(int parm, int n)
     }
 }
 
+/*
+ * Convert a termcap string to terminfo format.
+ * 'cap' is the relevant terminfo capability index.
+ * 's' is the string value of the capability.
+ * 'parameterized' tells what type of translations to do:
+ *	% translations if 1
+ *	pad translations if >=0
+ */
 char *
-_nc_captoinfo(
-/* convert a termcap string to terminfo format */
-    register const char *cap,	/* relevant terminfo capability index */
-    register const char *s,	/* string value of the capability */
-    int const parametrized	/* do % translations if 1, pad translations if >=0 */
-)
+_nc_captoinfo(const char *cap, const char *s, int const parameterized)
 {
     const char *capstart;
 
@@ -291,16 +294,16 @@ _nc_captoinfo(
     capstart = 0;
     if (s == 0)
 	s = "";
-    if (parametrized >= 0 && isdigit(*s))
+    if (parameterized >= 0 && isdigit(UChar(*s)))
 	for (capstart = s;; s++)
-	    if (!(isdigit(*s) || *s == '*' || *s == '.'))
+	    if (!(isdigit(UChar(*s)) || *s == '*' || *s == '.'))
 		break;
 
     while (*s != '\0') {
 	switch (*s) {
 	case '%':
 	    s++;
-	    if (parametrized < 1) {
+	    if (parameterized < 1) {
 		dp = save_char(dp, '%');
 		break;
 	    }
@@ -349,7 +352,7 @@ _nc_captoinfo(
 		break;
 	    case 'a':
 		if ((*s == '=' || *s == '+' || *s == '-'
-			|| *s == '*' || *s == '/')
+		     || *s == '*' || *s == '/')
 		    && (s[1] == 'p' || s[1] == 'c')
 		    && s[2] != '\0') {
 		    int l;
@@ -461,7 +464,7 @@ _nc_captoinfo(
 		dp = save_char(dp, '%');
 		s--;
 		_nc_warning("unknown %% code %s (%#x) in %s",
-		    unctrl(*s), (*s) & 0xff, cap);
+			    unctrl((chtype) *s), UChar(*s), cap);
 		break;
 	    }
 	    break;
@@ -542,7 +545,7 @@ _nc_captoinfo(
     if (capstart) {
 	dp = save_string(dp, "$<");
 	for (s = capstart;; s++)
-	    if (isdigit(*s) || *s == '*' || *s == '.')
+	    if (isdigit(UChar(*s)) || *s == '*' || *s == '.')
 		dp = save_char(dp, *s);
 	    else
 		break;
@@ -566,8 +569,8 @@ bcd_expression(const char *str)
     char ch1, ch2;
 
     if (sscanf(str, fmt, &ch1, &ch2) == 2
-	&& isdigit(ch1)
-	&& isdigit(ch2)
+	&& isdigit(UChar(ch1))
+	&& isdigit(UChar(ch2))
 	&& (ch1 == ch2)) {
 	len = 28;
 #ifndef NDEBUG
@@ -594,7 +597,7 @@ save_tc_char(char *bufptr, int c1)
 	bufptr = save_char(bufptr, c1);
     } else {
 	if (c1 == (c1 & 0x1f))	/* iscntrl() returns T on 255 */
-	    (void) strcpy(temp, unctrl(c1));
+	    (void) strcpy(temp, unctrl((chtype) c1));
 	else
 	    (void) sprintf(temp, "\\%03o", c1);
 	bufptr = save_string(bufptr, temp);
@@ -629,13 +632,12 @@ save_tc_inequality(char *bufptr, int c1, int c2)
  *     %m       exclusive-or all parameters with 0177 (not in 4.4BSD)
  */
 
+/*
+ * Convert a terminfo string to termcap format.  Parameters are as in
+ * _nc_captoinfo().
+ */
 char *
-_nc_infotocap(
-/* convert a terminfo string to termcap format */
-    register const char *cap GCC_UNUSED,	/* relevant termcap capability index */
-    register const char *str,	/* string value of the capability */
-    int const parametrized	/* do % translations if 1, pad translations if >=0 */
-)
+_nc_infotocap(const char *cap GCC_UNUSED, const char *str, int const parameterized)
 {
     int seenone = 0, seentwo = 0, saw_m = 0, saw_n = 0;
     const char *padding;
@@ -647,15 +649,15 @@ _nc_infotocap(
 
     /* we may have to move some trailing mandatory padding up front */
     padding = str + strlen(str) - 1;
-    if (*padding == '>' && *--padding == '/') {
+    if (padding > str && *padding == '>' && *--padding == '/') {
 	--padding;
-	while (isdigit(*padding) || *padding == '.' || *padding == '*')
+	while (isdigit(UChar(*padding)) || *padding == '.' || *padding == '*')
 	    padding--;
-	if (*padding == '<' && *--padding == '$')
+	if (padding > str && *padding == '<' && *--padding == '$')
 	    trimmed = padding;
 	padding += 2;
 
-	while (isdigit(*padding) || *padding == '.' || *padding == '*')
+	while (isdigit(UChar(*padding)) || *padding == '.' || *padding == '*')
 	    bufptr = save_char(bufptr, *padding++);
     }
 
@@ -667,13 +669,16 @@ _nc_infotocap(
 	    bufptr = save_char(bufptr, *++str);
 	} else if (str[0] == '$' && str[1] == '<') {	/* discard padding */
 	    str += 2;
-	    while (isdigit(*str) || *str == '.' || *str == '*' || *str ==
-		'/' || *str == '>')
+	    while (isdigit(UChar(*str))
+		   || *str == '.'
+		   || *str == '*'
+		   || *str == '/'
+		   || *str == '>')
 		str++;
 	    --str;
 	} else if (str[0] == '%' && str[1] == '%') {	/* escaped '%' */
 	    bufptr = save_string(bufptr, "%%");
-	} else if (*str != '%' || (parametrized < 1)) {
+	} else if (*str != '%' || (parameterized < 1)) {
 	    bufptr = save_char(bufptr, *str);
 	} else if (sscanf(str, "%%?%%{%d}%%>%%t%%{%d}%%+%%;", &c1, &c2) == 2) {
 	    str = strchr(str, ';');
@@ -692,7 +697,7 @@ _nc_infotocap(
 	    bufptr = save_string(bufptr, "%B");
 	} else if ((sscanf(str, "%%{%d}%%+%%c", &c1) == 1
 		    || sscanf(str, "%%'%c'%%+%%c", &ch1) == 1)
-	    && (cp = strchr(str, '+'))) {
+		   && (cp = strchr(str, '+'))) {
 	    str = cp + 2;
 	    bufptr = save_string(bufptr, "%+");
 
@@ -732,7 +737,7 @@ _nc_infotocap(
 	    case '8':
 	    case '9':
 		bufptr = save_char(bufptr, '%');
-		while (isdigit(*str))
+		while (isdigit(UChar(*str)))
 		    bufptr = save_char(bufptr, *str++);
 		if (strchr("doxX.", *str)) {
 		    if (*str != 'd')	/* termcap doesn't have octal, hex */
