@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -36,15 +36,20 @@
  */
 #include "panel.priv.h"
 
-MODULE_ID("$Id: p_replace.c,v 1.3 1999/09/29 15:22:32 juergen Exp $")
+MODULE_ID("$Id: p_replace.c,v 1.7 2001/02/24 23:41:38 tom Exp $")
 
-int
-replace_panel(PANEL *pan, WINDOW *win)
+NCURSES_EXPORT(int)
+replace_panel (PANEL *pan, WINDOW *win)
 {
   if(!pan)
     return(ERR);
-  if(_nc_panel_is_linked(pan))
+
+  if (IS_LINKED(pan)) {
+    Touchpan(pan);
     PANEL_UPDATE(pan,(PANEL*)0);
+  }
+  
   pan->win = win;
+
   return(OK);
 }
