@@ -1,12 +1,8 @@
 /*
- * $Id: rain.c,v 1.17 2001/09/15 21:46:35 tom Exp $
+ * $Id: rain.c,v 1.20 2002/06/29 23:32:18 tom Exp $
  */
-#include <signal.h>
-
 #include <test.priv.h>
 
-#include <term.h>		/* for tparm() */
-
 /* rain 11/3/1980 EPS/CITHEP */
 
 static float ranf(void);
@@ -39,6 +35,8 @@ main(
     float r;
     float c;
 
+    setlocale(LC_ALL, "");
+
     for (j = SIGHUP; j <= SIGTERM; j++)
 	if (signal(j, SIG_IGN) != SIG_IGN)
 	    signal(j, onsig);
